/* 
 * Copyright (c) 2012, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * */

/*
 *  ======== LoggerStreamer.c ========
 *  The following symbols must be defined
 *  GLOBAL_TIMESTAMP_ADRS : the address that EMUCNT0 can be read from.  If not defined, events will not be timestamped.
 */
#include <ti/uiaplus/loggers/stream/crc.h>
#include <ti/uiaplus/loggers/stream/crc.c> /* include the actual c code for this public domain software */

#include <xdc/std.h>

#include <xdc/runtime/Types.h>
#include <xdc/runtime/Log.h>
#include <xdc/runtime/Diags.h>
#include <xdc/runtime/Gate.h>
#include <ti/uia/runtime/UIAPacket.h>
#include <ti/uia/runtime/EventHdr.h>
#include <ti/uiaplus/loggers/stream/LoggerStreamer.h>
#include <ti/uiaplus/loggers/stream/LoggerStreamer_support.h>  /* externally declared functions provided by the application */
#include <string.h> // for setMask 


#ifdef xdc_target__isaCompatible_64P
#include <c6x.h>
#endif

/* Largest event size used to set the end pointer (threshold) */
#define LoggerStreamer_MAX_EVENT_SIZE (256)
#define LoggerStreamer_MAX_EVENT_SIZE_IN_WORDS (LoggerStreamer_MAX_EVENT_SIZE/sizeof(UInt32))
#define TIMESTAMP   (TRUE)
#define BITS8_IN_EVENTHDR           8
#define BITS8_IN_TIMESTAMP          ((TIMESTAMP & 1) << 3)
#define BITS8_IN_WRITEMEMORY        24
#define BITS8_IN_NUM_ARGS_0         0
#define BITS8_IN_NUM_ARGS_1         4
#define BITS8_IN_NUM_ARGS_2         8
#define BITS8_IN_NUM_ARGS_4         16
#define BITS8_IN_NUM_ARGS_8         32

#define BITS8_TO_BITS32(bits8)      ((bits8) / 4)
#define BITS32_TO_BITS8(bits32)     ((bits32) * 4)
#define MAU_TO_BITS32(mau)          ((mau) / sizeof(Bits32))
#define MAU_TO_BITS32_ROUND_UP(mau) (MAU_TO_BITS32((mau) + sizeof(Bits32) - 1))
#define BITS32_TO_MAU(bits32)       ((bits32) * sizeof(Bits32))
#define BITS8_TO_MAU(bits8)         (BITS32_TO_MAU(BITS8_TO_BITS32(bits8)))
#define MAU_TO_BITS8(mau)           (BITS32_TO_BITS8(MAU_TO_BITS32(mau)))
#define LoggerStreamer_HDR_OFFSET_IN_WORDS 4

/* Generate the event length based on write size and timestamp */
#define LoggerStreamer_GET_LENGTH_IN_BYTES(size, timestamp) size + timestamp
#define BYTES_IN_WORD 4

UInt16 LoggerStreamer_eventRecSequence = 0;

UInt16 LoggerStreamer_uiaPacketSequence = 0;

#define HDR_OFFSET_IN_WORDS 4        /* Size of the UIA header */
#define BYTES_IN_WORD 4

volatile UInt32 UIALogger_Diags_RuntimeOn = 0x0ffff;

typedef struct _LoggerStreamer_module_State {
	xdc_Bool enabled;
	xdc_UInt32* buffer;
	xdc_UInt32* write;
	xdc_UInt32* end;
	xdc_UInt32 numDroppedEvents;
	xdc_UInt32 bufSizeInWords;
} LoggerStreamer_module_State;

LoggerStreamer_module_State LoggerStreamer_ModuleState;
LoggerStreamer_module_State* LoggerStreamer_module = &LoggerStreamer_ModuleState;
xdc_Bool LoggerStreamer_gIsInitialized = FALSE;


//#define LoggerStreamer_BUFFER_SIZE (1024)

#ifdef GLOBAL_TIMESTAMP_ADRS
#define LoggerStreamer_isTimestampEnabled (TRUE)
#else
#define LoggerStreamer_isTimestampEnabled (FALSE)
#endif

Void LoggerStreamer_init(UInt32* buffer1, UInt32 bufSizeInMAUs);
Void LoggerStreamer_flush();
Void UIALogger_setMask(String src);
crc LoggerStreamer_generateCRC(unsigned char const message[], int nBytes);
/* Init LoggerFlexSupport module */
Void LoggerFlexSupport_init(Void);

/* Exit LoggerFlexSupport module */
Void LoggerFlexSupport_exit(Void);

/* write a logger event which has no arguments */
Void LoggerFlexSupport_write0(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid);

/* write a logger event which has 1 argument */
Void LoggerFlexSupport_write1(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid, IArg a1);

/* write a logger event which has 2 arguments */
Void LoggerFlexSupport_write2(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid, IArg a1, 
    IArg a2);

/* write a logger event which has 4 arguments */
Void LoggerFlexSupport_write4(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid, IArg a1, 
    IArg a2, IArg a3, IArg a4);

/* write a logger event which has 8 arguments */
Void LoggerFlexSupport_write8(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid, IArg a1, 
    IArg a2, IArg a3, IArg a4, IArg a5, IArg a6, IArg a7, IArg a8);

Void LoggerFlexSupport_writeMemoryRange(xdc_runtime_Log_Event evt,
		xdc_runtime_Types_ModuleId mid, UInt32 snapshotId,IArg fileName, IArg lineNum, IArg fmt,
		IArg startAdrs, UInt32 lengthInMAUs);
        
/* Disable logging */
Bool LoggerFlexSupport_disable();

/* Enable logging */
Bool LoggerFlexSupport_enable();

/* Init LoggerFlexSupport module.
 * NOTE: logger initialization must be done
 * explicitly by the application calling the
 * LoggerStreamer_init API and providing a buffer 
 * to log events to.
 */
Void LoggerFlexSupport_init() {
}

/* Exit LoggerFlexSupport module */
Void LoggerFlexSupport_exit() {
     LoggerStreamer_ModuleState.enabled = FALSE;
}
/*
 *  ======== LoggerStreamer_init ========
 *  Assigns an initialized buffer that LoggerStreamer can store events in,
 *  of size bufSizeInMAUs.  
 *  buffer1:       Pointer to the start of the initialized buffer that LoggerStreamer will 
 *                 fill with Log events.  Buffer should have previously been initialized by
 *                 passing it to the initBuffer API.
 *
 *  bufSizeInMAUs: The length of the buffer1 buffer, including the length of the UIAPacket_Hdr
 *                 data structure. 
 */
Void LoggerStreamer_init(UInt32* buffer1,UInt32 bufSizeInMAUs)
{
	LoggerStreamer_ModuleState.buffer = buffer1;
	LoggerStreamer_ModuleState.enabled = TRUE;
	LoggerStreamer_ModuleState.numDroppedEvents = 0;
	LoggerStreamer_ModuleState.write = LoggerStreamer_ModuleState.buffer + HDR_OFFSET_IN_WORDS;
	LoggerStreamer_ModuleState.bufSizeInWords = bufSizeInMAUs/sizeof(UInt32);
	LoggerStreamer_ModuleState.end = LoggerStreamer_ModuleState.buffer + LoggerStreamer_ModuleState.bufSizeInWords;
	LoggerStreamer_gIsInitialized = TRUE;
}

/*
 *  ======== LoggerStreamer_flush =========
 *  If there is data in the buffer, call the exchange function.
 */
Void LoggerStreamer_flush()
{
	if (LoggerStreamer_gIsInitialized) {
		/* If there is data in the buffer */
		if (LoggerStreamer_ModuleState.write
				> LoggerStreamer_ModuleState.buffer + HDR_OFFSET_IN_WORDS) {
			/* Set UIA packet length and sequence number */
			UIAPacket_setEventLength((UIAPacket_Hdr*)LoggerStreamer_ModuleState.buffer,
					(Bits32)((LoggerStreamer_ModuleState.write
							- LoggerStreamer_ModuleState.buffer) * BYTES_IN_WORD));
			UIAPacket_setSequenceCount((UIAPacket_Hdr*)LoggerStreamer_ModuleState.buffer,
					LoggerStreamer_uiaPacketSequence);

			/*
			 *  Whe the application calls flush there will always be empty space
			 *  at the end of the buffer. To let UIA know it should ignore this we
			 *  add a 32 bit Invalid UIA header with the length of the empty space.
			 */
			UIAPacket_setInvalidHdr(LoggerStreamer_ModuleState.write,
					(LoggerStreamer_ModuleState.end
							- LoggerStreamer_ModuleState.write) * BYTES_IN_WORD);

			/* Set the module write ptr to NULL to prevent log calls in exchange */
			LoggerStreamer_ModuleState.write = NULL;

			/* Send filled buffer to exchange function */
			LoggerStreamer_ModuleState.buffer =
					(UInt32*)bufferExchange((Ptr)LoggerStreamer_ModuleState.buffer);

			/* Update ptrs to new buffer */
			LoggerStreamer_ModuleState.write =
					LoggerStreamer_ModuleState.buffer + HDR_OFFSET_IN_WORDS;
			LoggerStreamer_ModuleState.end =
					LoggerStreamer_ModuleState.buffer + LoggerStreamer_ModuleState.bufSizeInWords;
			LoggerStreamer_uiaPacketSequence++;
		}
	}
}

/*
 *  ======== LoggerFlexSupport_disable ========
 */
Bool LoggerFlexSupport_disable()
{
	Bool prev;

	UInt key = Gate_enterSystem();
	prev = LoggerStreamer_ModuleState.enabled;
	LoggerStreamer_ModuleState.enabled = FALSE;
	Gate_leaveSystem(key);

	return (prev);
}

/*
 *  ======== LoggerStreamer_enable ========
 */
Bool LoggerFlexSupport_enable()
{
	Bool prev;

	UInt key = Gate_enterSystem();
	prev = LoggerStreamer_ModuleState.enabled;
	LoggerStreamer_ModuleState.enabled = TRUE;
	Gate_leaveSystem(key);

	return (prev);
}


/*
 *  ======== LoggerStreamer_fillInTimestamp =========
 */

static inline UInt32* LoggerStreamer_fillInTimestamp(UInt32 *writePtr)
{
	getTimestamp(writePtr,(writePtr+1));
	return (writePtr+2);
}

/*
 *  ======== LoggerStreamer2_fillInEventHdrSnapshot =========
 */
static inline UInt32* LoggerStreamer_fillInEventHdrSnapshot(UInt32 *writePtr,
		SizeT numBits8ToWrite, Bits32 serial)
{
	if (TIMESTAMP) {
		*(writePtr++) = EventHdr_genEventHdrWord1(numBits8ToWrite, serial,
				EventHdr_HdrType_EventWithSnapshotIdAndTimestamp);
		writePtr = LoggerStreamer_fillInTimestamp(writePtr);
	}
	else {
		*(writePtr++) = EventHdr_genEventHdrWord1(numBits8ToWrite, serial,
				EventHdr_HdrType_EventWithSnapshotId);
	}
	return (writePtr);
}

/*
 *  ======== UIALogger_write0 =========
 */
Void LoggerFlexSupport_write0(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid)
{
	UInt key;
	UInt32* writePtr = LoggerStreamer_ModuleState.write;
	UInt numBytesToWrite = LoggerStreamer_GET_LENGTH_IN_BYTES(
							LoggerStreamer_WRITE0_SIZE_IN_BYTES,
							LoggerStreamer_TIMESTAMP);
	if ((writePtr != NULL)&&(LoggerStreamer_gIsInitialized)&&(LoggerStreamer_ModuleState.enabled)) {


		key = Gate_enterSystem();

		LoggerStreamer_eventRecSequence++;

		if ((writePtr + MAU_TO_BITS32(numBytesToWrite)) >= LoggerStreamer_ModuleState.end) {
			LoggerStreamer_flush();
			writePtr = LoggerStreamer_ModuleState.write;
		}

		*writePtr++ = ti_uia_runtime_EventHdr_genEventHdrWord1(
				LoggerStreamer_GET_LENGTH_IN_BYTES(
						LoggerStreamer_WRITE0_SIZE_IN_BYTES,
						LoggerStreamer_TIMESTAMP),
						LoggerStreamer_eventRecSequence,
						ti_uia_runtime_EventHdr_HdrType_EventWithTimestamp);
		writePtr = LoggerStreamer_fillInTimestamp(writePtr);

		*writePtr++ = xdc_runtime_Types_makeEvent(xdc_runtime_Log_getEventId(evt),
				mid);

		LoggerStreamer_ModuleState.write = writePtr;

		Gate_leaveSystem(key);
	}
}

/*
 *  ======== UIALogger_write1 =========
 */
Void LoggerFlexSupport_write1(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid, IArg a1)
{
	UInt key;
	UInt32* writePtr = LoggerStreamer_ModuleState.write;
	UInt numBytesToWrite = LoggerStreamer_GET_LENGTH_IN_BYTES(
							LoggerStreamer_WRITE1_SIZE_IN_BYTES,
							LoggerStreamer_TIMESTAMP);
	if ((writePtr != NULL)&&(LoggerStreamer_gIsInitialized)&&(LoggerStreamer_ModuleState.enabled)) {


		key = Gate_enterSystem();

		LoggerStreamer_eventRecSequence++;

		if ((writePtr + MAU_TO_BITS32(numBytesToWrite)) >= LoggerStreamer_ModuleState.end) {
			LoggerStreamer_flush();
			writePtr = LoggerStreamer_ModuleState.write;
		}

		*writePtr++ = ti_uia_runtime_EventHdr_genEventHdrWord1(
				LoggerStreamer_GET_LENGTH_IN_BYTES(
						LoggerStreamer_WRITE1_SIZE_IN_BYTES,
						LoggerStreamer_TIMESTAMP),
						LoggerStreamer_eventRecSequence,
						ti_uia_runtime_EventHdr_HdrType_EventWithTimestamp);
		writePtr = LoggerStreamer_fillInTimestamp(writePtr);

		*writePtr++ = xdc_runtime_Types_makeEvent(xdc_runtime_Log_getEventId(evt),
				mid);
		*writePtr++ = a1;

		LoggerStreamer_ModuleState.write = writePtr;

		Gate_leaveSystem(key);
	}
}

/*
 *  ======== UIALogger_write2 =========
 */
Void LoggerFlexSupport_write2(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid, IArg a1, IArg a2)
{
	UInt key;
	UInt32* writePtr = LoggerStreamer_ModuleState.write;
	UInt numBytesToWrite = LoggerStreamer_GET_LENGTH_IN_BYTES(
							LoggerStreamer_WRITE2_SIZE_IN_BYTES,
							LoggerStreamer_TIMESTAMP);
	if ((writePtr != NULL)&&(LoggerStreamer_gIsInitialized)&&(LoggerStreamer_ModuleState.enabled)) {

		key = Gate_enterSystem();

		LoggerStreamer_eventRecSequence++;

		if ((writePtr + MAU_TO_BITS32(numBytesToWrite)) >= LoggerStreamer_ModuleState.end) {
			LoggerStreamer_flush();
			writePtr = LoggerStreamer_ModuleState.write;
		}

		*writePtr++ = ti_uia_runtime_EventHdr_genEventHdrWord1(
				LoggerStreamer_GET_LENGTH_IN_BYTES(
						LoggerStreamer_WRITE2_SIZE_IN_BYTES,
						LoggerStreamer_TIMESTAMP),
						LoggerStreamer_eventRecSequence,
						ti_uia_runtime_EventHdr_HdrType_EventWithTimestamp);
		writePtr = LoggerStreamer_fillInTimestamp(writePtr);

		*writePtr++ = xdc_runtime_Types_makeEvent(xdc_runtime_Log_getEventId(evt),
				mid);
		*writePtr++ = a1;
		*writePtr++ = a2;

		LoggerStreamer_ModuleState.write = writePtr;

		Gate_leaveSystem(key);
	}
}

/*
 *  ======== UIALogger_write4 =========
 */
Void LoggerFlexSupport_write4(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid,
           IArg a1, IArg a2, IArg a3, IArg a4)
{
	UInt key;
	UInt32* writePtr = LoggerStreamer_ModuleState.write;
	UInt numBytesToWrite = LoggerStreamer_GET_LENGTH_IN_BYTES(
							LoggerStreamer_WRITE4_SIZE_IN_BYTES,
							LoggerStreamer_TIMESTAMP);
	if ((writePtr != NULL)&&(LoggerStreamer_gIsInitialized)&&(LoggerStreamer_ModuleState.enabled)) {

		key = Gate_enterSystem();

		LoggerStreamer_eventRecSequence++;

		if ((writePtr + MAU_TO_BITS32(numBytesToWrite)) >= LoggerStreamer_ModuleState.end) {
			LoggerStreamer_flush();
			writePtr = LoggerStreamer_ModuleState.write;
		}

		*writePtr++ = ti_uia_runtime_EventHdr_genEventHdrWord1(
				LoggerStreamer_GET_LENGTH_IN_BYTES(LoggerStreamer_WRITE4_SIZE_IN_BYTES, LoggerStreamer_TIMESTAMP),
				LoggerStreamer_eventRecSequence,
				ti_uia_runtime_EventHdr_HdrType_EventWithTimestamp);
		writePtr = LoggerStreamer_fillInTimestamp(writePtr);

		*writePtr++ = xdc_runtime_Types_makeEvent(xdc_runtime_Log_getEventId(evt),
				mid);
		*writePtr++ = a1;
		*writePtr++ = a2;
		*writePtr++ = a3;
		*writePtr++ = a4;

		LoggerStreamer_ModuleState.write = writePtr;

		Gate_leaveSystem(key);
	}
}


/*
 *  ======== UIALogger_write8 =========
 */
Void LoggerFlexSupport_write8(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid,
		IArg a1, IArg a2, IArg a3, IArg a4,	IArg a5, IArg a6, IArg a7, IArg a8)
{
	UInt key;
	UInt32* writePtr = LoggerStreamer_ModuleState.write;
	UInt numBytesToWrite = LoggerStreamer_GET_LENGTH_IN_BYTES(
							LoggerStreamer_WRITE8_SIZE_IN_BYTES,
							LoggerStreamer_TIMESTAMP);
	if ((writePtr != NULL)&&(LoggerStreamer_gIsInitialized)&&(LoggerStreamer_ModuleState.enabled)) {

		key = Gate_enterSystem();

		LoggerStreamer_eventRecSequence++;

		if ((writePtr + MAU_TO_BITS32(numBytesToWrite)) >= LoggerStreamer_ModuleState.end) {
			LoggerStreamer_flush();
			writePtr = LoggerStreamer_ModuleState.write;
		}

		*writePtr++ = ti_uia_runtime_EventHdr_genEventHdrWord1(
				LoggerStreamer_GET_LENGTH_IN_BYTES(LoggerStreamer_WRITE8_SIZE_IN_BYTES, LoggerStreamer_TIMESTAMP),
				LoggerStreamer_eventRecSequence,
				ti_uia_runtime_EventHdr_HdrType_EventWithTimestamp);
		writePtr = LoggerStreamer_fillInTimestamp(writePtr);

		*writePtr++ = xdc_runtime_Types_makeEvent(xdc_runtime_Log_getEventId(evt),
				mid);
		*writePtr++ = a1;
		*writePtr++ = a2;
		*writePtr++ = a3;
		*writePtr++ = a4;
		*writePtr++ = a5;
		*writePtr++ = a6;
		*writePtr++ = a7;
		*writePtr++ = a8;

		LoggerStreamer_ModuleState.write = writePtr;

		Gate_leaveSystem(key);
	}
}


/*  ======== UIALogger_writeMemoryRange =========
 *  logs a range of consecutive memory values.
 *
 *  Breaks up large blocks into a series of smaller blocks in order to limit the
 *  maximum amount of time that interrupts are disabled.
 */
Void LoggerFlexSupport_writeMemoryRange(xdc_runtime_Log_Event evt,
		xdc_runtime_Types_ModuleId mid, UInt32 snapshotId,
		IArg fileName, IArg lineNum, IArg fmt,
		IArg startAdrs, UInt32 lengthInMAUs)
{
	Int i;
	SizeT eventSizeBits32;
	SizeT dataSizeBits32;
	Ptr pAdrs = (Ptr)startAdrs;
	SizeT headerSizeBits32;
	SizeT lenAdjustedBits32;
	Int numRecords;
	UInt key;
	UInt32* writePtr = LoggerStreamer_ModuleState.write;

	if ((writePtr != NULL)&&(LoggerStreamer_gIsInitialized)&&(LoggerStreamer_ModuleState.enabled)) {

		headerSizeBits32 = BITS8_TO_BITS32(BITS8_IN_EVENTHDR +
				BITS8_IN_WRITEMEMORY + BITS8_IN_TIMESTAMP);
		lenAdjustedBits32 = MAU_TO_BITS32_ROUND_UP(lengthInMAUs);
		numRecords = (lenAdjustedBits32 / (LoggerStreamer_MAX_EVENT_SIZE_IN_WORDS - headerSizeBits32)) + 1;

		for (i = 0; i < numRecords; i++) {

			if ((headerSizeBits32 + lenAdjustedBits32) > LoggerStreamer_MAX_EVENT_SIZE_IN_WORDS) {
				dataSizeBits32 = LoggerStreamer_MAX_EVENT_SIZE_IN_WORDS - headerSizeBits32;
				lenAdjustedBits32 -= dataSizeBits32;
			}
			else {
				dataSizeBits32 = lenAdjustedBits32;
			}
			eventSizeBits32 = dataSizeBits32 + headerSizeBits32;

			key = Gate_enterSystem();
			/*
			 * Update serial number even if the buffer is full or disabled.
			 * We do this because a reader (decoder) of the buffer needs to
			 * know if events have been missed, and the buffer might become
			 * un-FULL at some later time.
			 */
	         LoggerStreamer_eventRecSequence++;
			if ((writePtr + eventSizeBits32) >= LoggerStreamer_ModuleState.end) {
				LoggerStreamer_flush();
				writePtr = LoggerStreamer_ModuleState.write;
			}
			if (writePtr == NULL ) {


				Gate_leaveSystem(key);

				//TODO: log the first 8 bytes of the event to the overflow buffer
				LoggerStreamer_ModuleState.numDroppedEvents++;
			}
			else {

				writePtr = LoggerStreamer_fillInEventHdrSnapshot((Bits32 *)LoggerStreamer_ModuleState.write, 
                                                BITS32_TO_BITS8(eventSizeBits32),		
						LoggerStreamer_eventRecSequence);
				*(writePtr++) = ((evt) & 0xffff0000) | mid;
				*(writePtr++) = fileName;
				*(writePtr++) = lineNum;
				*(writePtr++) = snapshotId;
				*(writePtr++) = (Bits32)pAdrs;
				*(writePtr++) = (((BITS32_TO_MAU(dataSizeBits32) & 0x0FFFF)<<16) |
						(lengthInMAUs & 0x0FFFF));
				*(writePtr++) = fmt;

				memcpy(writePtr, (Bits32 *)pAdrs, BITS32_TO_MAU(dataSizeBits32));
				writePtr = (UInt32*)((Bits32)writePtr + BITS32_TO_MAU(dataSizeBits32));
				LoggerStreamer_ModuleState.write = writePtr;
				pAdrs = (Ptr)((Bits32)pAdrs + BITS32_TO_MAU(dataSizeBits32));
				Gate_leaveSystem(key);
			}
		}
		LoggerStreamer_ModuleState.write = writePtr;

	}

}


#define MASKOPS "=+-"

/*
 *  ======== setMask ========
 */
Void UIALogger_setMask(String src)
{

	Bits16 mask;
	Int len;
	String ops;
	Char op;
	Char bc;
	Char *bcp;

	for (;;) {

		/* find first/next op in the string src */
		ops = strpbrk(src, MASKOPS);
		if (ops == NULL) {
			return;
		}

		/* if the module specification is "", return */
		len = ops - src;    /* len = length of the module specification */
		if (len == 0) {
			return;
		}

		/* get operation from ops string */
		bcp = ops;
		op = *bcp++;    /* op = one of the ops in MASKOPS */

		/* compute mask from ops string */
		mask = 0;
		for (;;) {
			bc = *bcp++;
			if (bc == ';' || bc == '\0') {
				break;
			}
			else if (bc >= '1' && bc <= '8') {
				mask |= 0x100 << (bc - '1');
			}
			else if (bc == 'L') {
				mask |= Diags_LIFECYCLE;
			}
			else if (bc == 'E') {
				mask |= Diags_ENTRY;
			}
			else if (bc == 'X') {
				mask |= Diags_EXIT;
			}
			else if (bc == 'A') {
				mask |= Diags_ASSERT;
			}
			else if (bc == 'I') {
				mask |= Diags_INTERNAL;
			}
			else if (bc == 'Z') {
				mask |= Diags_ANALYSIS;
			}
			else if (bc == 'F') {
				mask |= Diags_INFO;
			}
			else if (bc == 'S') {
				mask |= Diags_STATUS;
			}
			else {
				return;
			}
		}

		/* Update the module's mask */
		if (op == '+') {
			UIALogger_Diags_RuntimeOn |= mask;
		}
		else if (op == '-') {
			UIALogger_Diags_RuntimeOn &= ~mask;
		}
		else {
			UIALogger_Diags_RuntimeOn = mask;
		}


		/* if we reached the end of the string, return */
		if (bc == '\0') {
			return;
		}

		/* otherwise, move src to end of the operation string and restart */
		src = bcp;
	}
}

crc LoggerStreamer_generateCRC(unsigned char const message[], int nBytes){
	return(crcSlow(message,nBytes));
}

